/*
Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package traitement;

import java.awt.*;
import java.awt.event.*;
import ptolemy.plot.Plot;


/**
 * Affichage d'un graphe dans une fentre, avec des valeurs donnes
 */
public class FenetreGraphe extends Frame {
    
    Label labcoord, labinfo;
    Plot plot;
    String labelx, labely;
    
    double[] vx;
    double[] vy;
    
    public FenetreGraphe(double[] vx, double[] vy, String labelx, String labely) {
        super(Traitement.rb.getString("Graphe"));
        this.vx = vx;
        this.vy = vy;
        this.labelx = labelx;
        this.labely = labely;
        
        setLayout(new BorderLayout());
        plot = new Plot();
        add(plot, BorderLayout.CENTER);
        plot.setMarksStyle("none");
        plot.setConnected(true);
        if (labelx != null)
            plot.setXLabel(labelx);
        else if (labely != null)
            plot.setXLabel(labely);
        for (int i=0; i<vx.length; i++)
            plot.addPoint(0, vx[i], vy[i], true);
        plot.addMouseMotionListener(new MML());
        
        Panel pbas = new Panel(new GridLayout(3, 1));
        
        labcoord = new Label();
        pbas.add(labcoord);
        
        double somme = 0;
        for (int i=0; i<vy.length; i++)
            somme += vy[i];
        double moyenne = somme / vy.length;
        somme = 0;
        for (int i=0; i<vy.length; i++)
            somme += Math.abs(vy[i] - moyenne);
        double ecart = somme / vy.length;
        labinfo = new Label(Traitement.rb.getString("moyenne") + " " + joliFormattage(moyenne, 4) +
            "   " + Traitement.rb.getString("cart-type") + " " + joliFormattage(ecart, 4));
        pbas.add(labinfo);
        
        Panel pfit = new Panel(new FlowLayout());
        Button btableau = new Button(Traitement.rb.getString("Fermer"));
        btableau.addActionListener(new ActionAdapter() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });
        pfit.add(btableau);
        pbas.add(pfit);
        
        add(pbas, BorderLayout.SOUTH);
        
        plot.setVisible(true);
        setSize(500, 400);
        
        Dimension dim = getSize();
        Dimension ecran = getToolkit().getScreenSize();
        setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        
        validate();
    }
    
    public abstract class ActionAdapter extends Object implements ActionListener {
        public abstract void actionPerformed(ActionEvent ae);
    }
    
    class MML implements MouseMotionListener {
        public void mouseDragged(MouseEvent e) {
        }
        public void mouseMoved(MouseEvent e) {
            double coords[] = plot.getPointCoordinates(e.getX(), e.getY());
            labcoord.setText("x=" + joliFormattage(coords[0], 4) +
                "  y=" + joliFormattage(coords[1], 4));
            validate();
        }
    }
    
    public static String joliFormattage(double d, int ndecimales) {
        int exposant;
        double ddec = Math.pow(10, ndecimales);
        if (d != 0)
            exposant = (int)Math.floor(Math.log(Math.abs(d))/Math.log(10));
        else
            exposant = 0;
        double mantisse = d / (Math.pow(10,exposant));
        String s;
        if (exposant == 0) {
            mantisse = Math.round(mantisse*ddec)/ddec;
            if (mantisse - (int)(mantisse) < 1E-8)
                s = ""+(int)mantisse;
            else
                s = ""+mantisse;
        } else if (exposant == 1) {
            mantisse = Math.round(mantisse*ddec)/(ddec/10);
            if (mantisse - (int)(mantisse) < 1E-8)
                s = ""+(int)mantisse;
            else
                s = ""+mantisse;
        } else if (exposant == 2) {
            mantisse = Math.round(mantisse*ddec)/(ddec/100);
            if (mantisse - (int)(mantisse) < 1E-8)
                s = ""+(int)mantisse;
            else
                s = ""+mantisse;
        } else {
            mantisse = Math.round(mantisse*ddec)/ddec;
            s = mantisse + "E" + exposant; // "E" prend moins de place que ".10^", important en abcisse
        }
        return(s);
    }
}
