/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package traitement;

import java.awt.Image;
import java.awt.image.MemoryImageSource;

public class ImageMemoire implements ImageIntf {
    
    private Traitement app;
    private int dimx=0, dimy=0;
    private int seuil_inf=0, seuil_sup=1000;
    private Image img = null;
    private int[][] valeurs;
    private String nom;
    
    public ImageMemoire(String nom, int[][] valeurs, Traitement app) {
        this.nom = nom;
        this.valeurs = valeurs;
        dimx = valeurs.length;
        dimy = valeurs[0].length;
        this.app = app;
    }
    
    public String getNom() {
        return(nom);
    }
    
    // creation de l'objet Image en fonction des seuils
    private void creationImage() {
        int[][] palette = app.getPalette();
        int[] pix = new int[dimx * dimy];
        int v;
        int k=0;
        for (int j=0; j<dimy; j++) {
            for (int i=0; i<dimx; i++) {
                v = (int)Math.round( 256 * ((double)(valeurs[i][j] - seuil_inf)) / (seuil_sup - seuil_inf + 1) );
                if (v < 0)
                    v = 0;
                else if (v > 255)
                    v = 255;
                //pix[k++] = (255 << 24) | (v << 16) | (v << 8) | v;
                pix[k++] = (255 << 24) | (palette[v][0] << 16) | (palette[v][1] << 8) | palette[v][2];
            }
        }
        if (img != null)
            img.flush();
        img = app.createImage(new MemoryImageSource(dimx, dimy, pix, 0, dimx));
    }
    
    public Image getImage() {
        if (img == null && valeurs != null)
            creationImage();
        return(img);
    }
    
    public int getSeuilInf() {
        return(seuil_inf);
    }
    
    public int getSeuilSup() {
        return(seuil_sup);
    }
    
    public void setSeuils(int mini, int maxi) {
        seuil_inf = mini;
        seuil_sup = maxi;
        creationImage();
    }
    
    public void recreer() {
        if (img != null)
            creationImage();
    }
    
    public int[][] getValeurs() {
        return(valeurs);
    }
}
