/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package traitement;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.net.*;
import java.util.ResourceBundle;
import java.util.Vector;

public class Traitement extends Applet implements ActionListener, ItemListener {
    public static ResourceBundle rb  = ResourceBundle.getBundle("traitement.LocalStrings");
    private List imgList;
    private Vector listeImages; // de ImageIntf
    private ImageIntf imageAffichee;
    private TextField tfSeuilBas, tfSeuilHaut;
    private Choice opChoice;
    private Choice palChoice;
    private ImageCanvas canvas;
    private Label coordLabel, infoLabel;
    private String opCourante = null;
    private int numImageCourante = -1;
    private int[] indices;
    private int[][] valeursCalcul;
    private Point ptRef;
    private Button bchanger, bauto;
    private int[][] palette = null;
    
    public void init() {
        creerPalette(rb.getString("niveaux_de_gris"));
        String surl;
        int n = 0;
        listeImages = new Vector();
        while (getParameter("image" + (n+1)) != null) {
            n++;
            try {
                listeImages.addElement(new ImageFITS(new URL(getDocumentBase(), getParameter("image" + n)), this));
            } catch (MalformedURLException ex) {
                System.err.println(ex.getClass().getName() + ": " + ex.getMessage());
                n--;
                break;
            }
        }
        setLayout(new BorderLayout());
        imgList = new List();
        for (int i=0; i<listeImages.size(); i++) {
            imgList.add(((ImageIntf)listeImages.elementAt(i)).getNom());
        }
        imgList.addItemListener(this);
        add(imgList, BorderLayout.WEST);
        
        Panel imgPanel = new Panel(new BorderLayout());
        canvas = new ImageCanvas(((ImageIntf)listeImages.elementAt(0)).getImage(), this);
        imgPanel.add(canvas, BorderLayout.CENTER);
        Panel seuilsPanel = new Panel();
        seuilsPanel.add(new Label(rb.getString("seuil_bas")));
        tfSeuilBas = new TextField(6);
        tfSeuilBas.addActionListener(this);
        seuilsPanel.add(tfSeuilBas);
        seuilsPanel.add(new Label(rb.getString("seuil_haut")));
        tfSeuilHaut = new TextField(6);
        tfSeuilHaut.addActionListener(this);
        seuilsPanel.add(tfSeuilHaut);
        bchanger = new Button(rb.getString("changer"));
        bchanger.setActionCommand("seuils");
        bchanger.addActionListener(this);
        seuilsPanel.add(bchanger);
        bauto = new Button(rb.getString("auto"));
        bauto.setActionCommand("auto");
        bauto.addActionListener(this);
        seuilsPanel.add(bauto);
        imgPanel.add(seuilsPanel, BorderLayout.SOUTH);
        add(imgPanel, BorderLayout.CENTER);
        
        Panel opinfoPanel = new Panel(new GridLayout(2,1));
        Panel opPanel = new Panel();
        coordLabel = new Label("                              ");
        opPanel.add(coordLabel);
        opChoice = new Choice();
        opChoice.add(rb.getString("opration"));
        opChoice.add(rb.getString("addition_avec_recentrage"));
        opChoice.add(rb.getString("addition_sans_recentrage"));
        opChoice.add(rb.getString("soustraction"));
        opChoice.add(rb.getString("division"));
        opChoice.add(rb.getString("coupe"));
        opChoice.add(rb.getString("filtre"));
        opChoice.add(rb.getString("supprimer"));
        opChoice.addItemListener(this);
        opPanel.add(opChoice);
        opPanel.add(new Label(rb.getString("palette")));
        palChoice = new Choice();
        palChoice.add(rb.getString("niveaux_de_gris"));
        palChoice.add(rb.getString("arc-en-ciel"));
        palChoice.add(rb.getString("rouges"));
        palChoice.add(rb.getString("verts"));
        palChoice.add(rb.getString("bleus"));
        palChoice.addItemListener(this);
        opPanel.add(palChoice);
        opinfoPanel.add(opPanel);
        infoLabel = new Label("");
        infoLabel.setForeground(Color.red);
        opinfoPanel.add(infoLabel);
        add(opinfoPanel, BorderLayout.SOUTH);
        
        canvas.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseMoved(MouseEvent e) {
                majCoord(e);
            }
            public void mouseDragged(MouseEvent e) {
                majCoord(e);
            }
            public void majCoord(MouseEvent e) {
                int[][] valeurs = imageAffichee.getValeurs();
                int dimx = valeurs.length;
                int dimy = valeurs[0].length;
                int x = e.getX();
                int y = e.getY();
                if (x >= 0 && x < dimx && y >= 0 && y < dimy) {
                    String texte = "("+(x+1)+","+(y+1)+") : " + valeurs[x][y];
                    if (texte.length() < 30)
                        for (int i=0; i<30-texte.length(); i++)
                            texte += "  ";
                    coordLabel.setText(texte);
                }
            }
        });
        
        if (listeImages.size() > 0) {
            imgList.select(0);
            ouvrir((ImageIntf)listeImages.elementAt(0));
        }
    }
    
    protected void ouvrir(ImageIntf im) {
        imageAffichee = im;
        Image img = imageAffichee.getImage();
        canvas.changerImage(img);
        tfSeuilBas.setText("" + imageAffichee.getSeuilInf());
        tfSeuilHaut.setText("" + imageAffichee.getSeuilSup());
    }
    
    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();
        if ("seuils".equals(cmd))
            changerSeuils();
        else if (e.getSource() == tfSeuilBas || e.getSource() == tfSeuilHaut)
            changerSeuils();
        else if ("auto".equals(cmd))
            seuilsAuto();
    }
    
    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED && e.getSource() == imgList)
            selection();
        else if (e.getSource() == opChoice && opChoice.getSelectedIndex() != 0) {
            operation(opChoice.getSelectedItem());
            opChoice.select(0);
        } else if (e.getSource() == palChoice)
            changerPalette(palChoice.getSelectedItem());
    }
    
    protected void operation(String nomOp) {
        if (rb.getString("addition_avec_recentrage").equals(nomOp))
            additionAvecRecentrage();
        else if (rb.getString("addition_sans_recentrage").equals(nomOp))
            additionSansRecentrage();
        else if (rb.getString("soustraction").equals(nomOp))
            soustraction();
        else if (rb.getString("division").equals(nomOp))
            division();
        else if (rb.getString("coupe").equals(nomOp))
            coupe();
        else if (rb.getString("filtre").equals(nomOp))
            filtre();
        else if (rb.getString("supprimer").equals(nomOp))
            supprimer();
    }
    
    protected void suiteOperation(Point p1, Point p2) {
        if ("addition".equals(opCourante))
            suiteAddition(p1, p2);
        else if (rb.getString("coupe").equals(opCourante))
            suiteCoupe(p1, p2);
    }
    
    protected void creerPalette(String choix) {
        if (palette == null)
            palette = new int[256][3];
        if (rb.getString("niveaux_de_gris").equals(choix)) {
            for (int i=0; i<256; i++) {
                palette[i][0] = i;
                palette[i][1] = i;
                palette[i][2] = i;
            }
        } else if (rb.getString("arc-en-ciel").equals(choix)) {
            // violet bleu vert jaune orange rouge
            for (int i=0; i<51; i++) {
                palette[i][0] = 102 - 2*i;
                palette[i][1] = 0;
                palette[i][2] = 255;
            }
            for (int i=51; i<102; i++) {
                palette[i][0] = 0;
                palette[i][1] = (i - 51)*5;
                palette[i][2] = 255 - (i - 51)*5;
            }
            for (int i=102; i<153; i++) {
                palette[i][0] = (i - 102)*5;
                palette[i][1] = 255;
                palette[i][2] = 0;
            }
            for (int i=153; i<204; i++) {
                palette[i][0] = 255;
                palette[i][1] = 255 - 2*(i - 153);
                palette[i][2] = 0;
            }
            for (int i=204; i<255; i++) {
                palette[i][0] = 255;
                palette[i][1] = 153 - 3*(i - 204);
                palette[i][2] = 0;
            }
            palette[255][0] = 255;
            palette[255][1] = 0;
            palette[255][2] = 0;
        } else if (rb.getString("rouges").equals(choix)) {
            for (int i=0; i<256; i++) {
                palette[i][0] = i;
                palette[i][1] = 0;
                palette[i][2] = 0;
            }
        } else if (rb.getString("verts").equals(choix)) {
            for (int i=0; i<256; i++) {
                palette[i][0] = 0;
                palette[i][1] = i;
                palette[i][2] = 0;
            }
        } else if (rb.getString("bleus").equals(choix)) {
            for (int i=0; i<256; i++) {
                palette[i][0] = 0;
                palette[i][1] = 0;
                palette[i][2] = i;
            }
        }
    }
    
    protected void changerPalette(String choix) {
        creerPalette(choix);
        for (int i=0; i<listeImages.size(); i++)
            ((ImageIntf)listeImages.elementAt(i)).recreer();
        Image img = imageAffichee.getImage();
        canvas.changerImage(img);
    }
    
    public int[][] getPalette() {
        return(palette);
    }
    
    protected void changerSeuils() {
        String sbas = tfSeuilBas.getText();
        String shaut = tfSeuilHaut.getText();
        int bas, haut;
        try {
            bas = Integer.valueOf(sbas).intValue();
            haut = Integer.valueOf(shaut).intValue();
        } catch (NumberFormatException ex) {
            alerte("Erreur de formattage: " + ex.getMessage());
            return;
        }
        imageAffichee.setSeuils(bas, haut);
        Image img = imageAffichee.getImage();
        canvas.changerImage(img);
    }
    
    protected void seuilsAuto() {
        int[][] valeurs = imageAffichee.getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        int mini=Integer.MAX_VALUE, maxi=0;
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++) {
                if (valeurs[i][j] < mini)
                    mini = valeurs[i][j];
                if (valeurs[i][j] > maxi)
                    maxi = valeurs[i][j];
            }
        int dimc = 100;
        int[] comptage = new int[dimc];
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++) {
                int posc = (int)Math.round((dimc - 1) * (Math.log((double)(valeurs[i][j] - mini + 1)) / Math.log(maxi - mini + 1)));
                comptage[posc]++;
            }
        double nb = 0;
        int imini = 0;
        while (nb < (((double)dimx) * dimy)*0.04) {
            nb += comptage[imini];
            imini++;
        }
        int bas = (int)Math.round(mini + Math.exp((((double)imini)/(dimc - 1)) * Math.log(maxi - mini)));
        nb = 0;
        int imaxi = 0;
        while (nb < (((double)dimx) * dimy)*0.99) {
            nb += comptage[imaxi];
            imaxi++;
        }
        int haut = (int)Math.round(mini + Math.exp((((double)imaxi)/(dimc - 1)) * Math.log(maxi - mini)));
        tfSeuilBas.setText(""+bas);
        tfSeuilHaut.setText(""+haut);
        imageAffichee.setSeuils(bas, haut);
        Image img = imageAffichee.getImage();
        canvas.changerImage(img);
    }
    
    protected void selection() {
        ouvrir((ImageIntf)listeImages.elementAt(imgList.getSelectedIndex()));
    }
    
    protected void alerte(String message) {
        Alerte f = new Alerte(chercherFrame(), rb.getString("Erreur"), message);
        Point apppoint = getLocationOnScreen();
        f.setLocation(apppoint.x+50, apppoint.y+50);
        f.setVisible(true);
    }
    
    protected Frame chercherFrame() {
        Container c = this; 
        while (c != null) { 
            if (c instanceof Frame) 
                return (Frame)c; 
            c = c.getParent(); 
        } 
        return (Frame)null; 
    }
    
    protected String demander(String message) {
        Question f = new Question(chercherFrame(), rb.getString("Question"), message);
        Point apppoint = getLocationOnScreen();
        f.setLocation(apppoint.x+50, apppoint.y+50);
        f.setVisible(true);
        return(f.getReponse());
    }
    
    protected int[] selectionImages(String question, int minsel, int maxsel) {
        SelectionDlg f = new SelectionDlg(chercherFrame(), question, minsel, maxsel, listeImages);
        Point apppoint = getLocationOnScreen();
        f.setLocation(apppoint.x+50, apppoint.y+50);
        f.setVisible(true);
        return(f.getIndices());
    }
    
    protected void additionAvecRecentrage() {
        indices = selectionImages(rb.getString("Slectionnez_addition"), 2, listeImages.size());
        if (indices == null)
            return;
        infoLabel.setText(rb.getString("Slectionnez_toile"));
        canvas.setOutil(ImageCanvas.CERCLE);
        opCourante = "addition";
        numImageCourante = 0;
        bchanger.setEnabled(false);
        ouvrir((ImageIntf)listeImages.elementAt(indices[numImageCourante]));
    }
    
    protected void suiteAddition(Point p1, Point p2) {
        int[][] valeurs = imageAffichee.getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        if (numImageCourante == 0) {
            ptRef = baryDisque(p1, p2, valeurs);
            valeursCalcul = new int[dimx][dimy];
            for (int i=0; i<dimx; i++)
                for (int j=0; j<dimy; j++)
                    valeursCalcul[i][j] = valeurs[i][j];
        } else {
            Point nouvPt = baryDisque(p1, p2, valeurs);
            for (int i=0; i<dimx; i++)
                for (int j=0; j<dimy; j++) {
                    int i2 = i + (nouvPt.x - ptRef.x);
                    int j2 = j + (nouvPt.y - ptRef.y);
                    if (i2 >= 0 && i2 < dimx && j2 >= 0 && j2 < dimy)
                        valeursCalcul[i][j] += valeurs[i2][j2];
                }
        }
        numImageCourante++;
        if (numImageCourante < indices.length) {
            imgList.select(indices[numImageCourante]);
            ouvrir((ImageIntf)listeImages.elementAt(indices[numImageCourante]));
            canvas.setOutil(ImageCanvas.CERCLE);
        } else
            finAddition();
    }
    
    protected void additionSansRecentrage() {
        indices = selectionImages(rb.getString("Slectionnez_addition"), 2, listeImages.size());
        if (indices == null)
            return;
        int[][] valeurs = ((ImageIntf)listeImages.elementAt(indices[0])).getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        valeursCalcul = new int[dimx][dimy];
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++)
                valeursCalcul[i][j] = valeurs[i][j];
        for (numImageCourante=1; numImageCourante < indices.length; numImageCourante++) {
            valeurs = ((ImageIntf)listeImages.elementAt(indices[numImageCourante])).getValeurs();
            for (int i=0; i<dimx; i++)
                for (int j=0; j<dimy; j++)
                    valeursCalcul[i][j] += valeurs[i][j];
        }
        finAddition();
    }
    
    // calcul du barycentre d'un ensemble de points  l'intrieur d'un disque dfinit par son centre p1 et un point p2
    protected Point baryDisque(Point p1, Point p2, int[][] valeurs) {
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        int rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
        int xmin = p1.x - rayon;
        if (xmin < 0)
            xmin = 0;
        int xmax = p1.x + rayon;
        if (xmax > dimx-1)
            xmax = dimx-1;
        int ymin = p1.y - rayon;
        if (ymin < 0)
            ymin = 0;
        int ymax = p1.y + rayon;
        if (ymax > dimy-1)
            ymax = dimy-1;
        double vx=0, vy=0;
        double totalx=0, totaly=0;
        for (int i=xmin; i<xmax; i++)
            for (int j=ymin; j<ymax; j++)
                if (Math.max(Math.abs(i - p1.x), Math.abs(j - p1.y)) <= rayon) {
                    vx += i*valeurs[i][j];
                    vy += j*valeurs[i][j];
                    totalx += valeurs[i][j];
                    totaly += valeurs[i][j];
                }
        int xres = (int)Math.round(vx / totalx);
        int yres = (int)Math.round(vy / totaly);
        return(new Point(xres, yres));
    }
    
    protected void finAddition() {
        infoLabel.setText("");
        bchanger.setEnabled(true);
        opCourante = null;
        numImageCourante = -1;
        canvas.setOutil(ImageCanvas.RIEN);
        int dimx = valeursCalcul.length;
        int dimy = valeursCalcul[0].length;
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++)
                valeursCalcul[i][j] = valeursCalcul[i][j] / indices.length;
        String nomNouvelleImage = demander(rb.getString("Nom_nouvelle_image"));
        if (nomNouvelleImage == null) {
            valeursCalcul = null;
            return;
        }
        ImageMemoire nouvelleImage = new ImageMemoire(nomNouvelleImage, valeursCalcul, this);
        nouvelleImage.setSeuils(imageAffichee.getSeuilInf(), imageAffichee.getSeuilSup());
        listeImages.addElement(nouvelleImage);
        imgList.add(nouvelleImage.getNom());
        imgList.select(imgList.getItemCount()-1);
        ouvrir(nouvelleImage);
        
        valeursCalcul = null;
    }
    
    protected void soustraction() {
        indices = selectionImages(rb.getString("Slectionnez_image1_soustraction"), 1, 1);
        if (indices == null)
            return;
        int ind1 = indices[0];
        indices = selectionImages(rb.getString("Slectionnez_image2"), 1, 1);
        if (indices == null)
            return;
        int ind2 = indices[0];
        int[][] valeurs = ((ImageIntf)listeImages.elementAt(ind1)).getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        valeursCalcul = new int[dimx][dimy];
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++)
                valeursCalcul[i][j] = valeurs[i][j];
        valeurs = ((ImageIntf)listeImages.elementAt(ind2)).getValeurs();
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++) {
                valeursCalcul[i][j] -= valeurs[i][j];
                if (valeursCalcul[i][j] < 0)
                    valeursCalcul[i][j] = 0;
            }
        String nomNouvelleImage = demander(rb.getString("Nom_nouvelle_image"));
        if (nomNouvelleImage == null) {
            valeursCalcul = null;
            return;
        }
        ImageMemoire nouvelleImage = new ImageMemoire(nomNouvelleImage, valeursCalcul, this);
        listeImages.addElement(nouvelleImage);
        imgList.add(nouvelleImage.getNom());
        imgList.select(imgList.getItemCount()-1);
        ouvrir(nouvelleImage);
        valeursCalcul = null;
    }
    
    protected void division() {
        indices = selectionImages(rb.getString("Slectionnez_image1_division"), 1, 1);
        if (indices == null)
            return;
        int ind1 = indices[0];
        indices = selectionImages(rb.getString("Slectionnez_image2"), 1, 1);
        if (indices == null)
            return;
        int ind2 = indices[0];
        int[][] valeurs = ((ImageIntf)listeImages.elementAt(ind1)).getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        valeursCalcul = new int[dimx][dimy];
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++)
                valeursCalcul[i][j] = valeurs[i][j];
        valeurs = ((ImageIntf)listeImages.elementAt(ind2)).getValeurs();
        double total = 0;
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++)
                total += valeurs[i][j];
        double moyenne = total / (((double)dimx) * dimy);
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++) {
                if (valeurs[i][j] != 0)
                    valeursCalcul[i][j] = (int)Math.round(valeursCalcul[i][j] / (valeurs[i][j] / moyenne));
                else
                    valeursCalcul[i][j] = 0;
            }
        String nomNouvelleImage = demander(rb.getString("Nom_nouvelle_image"));
        if (nomNouvelleImage == null) {
            valeursCalcul = null;
            return;
        }
        ImageMemoire nouvelleImage = new ImageMemoire(nomNouvelleImage, valeursCalcul, this);
        nouvelleImage.setSeuils(imageAffichee.getSeuilInf(), imageAffichee.getSeuilSup());
        listeImages.addElement(nouvelleImage);
        imgList.add(nouvelleImage.getNom());
        imgList.select(imgList.getItemCount()-1);
        ouvrir(nouvelleImage);
        valeursCalcul = null;
    }
    
    protected void coupe() {
        infoLabel.setText(rb.getString("Slectionnez_coupe"));
        canvas.setOutil(ImageCanvas.LIGNE);
        opCourante = rb.getString("coupe");
        bchanger.setEnabled(false);
    }
    
    protected void suiteCoupe(Point p1, Point p2) {
        infoLabel.setText("");
        bchanger.setEnabled(true);
        opCourante = null;
        canvas.setOutil(ImageCanvas.RIEN);
        int[][] valeurs = imageAffichee.getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        int x1 = p1.x;
        int x2 = p2.x;
        int y1 = p1.y;
        int y2 = p2.y;
        int dx = Math.abs(x2 - x1);
        int dy = Math.abs(y2 - y1);
        double[] vx;
        double[] vy;
        int x, y;
        if (dx > dy) {
            vx = new double[dx];
            vy = new double[dx];
            int sgn;
            if (x2 > x1)
                sgn = 1;
            else
                sgn = -1;
            for (int i=0; i<dx; i++) {
                x = x1 + sgn * i;
                y = y1 + (int)Math.round((y2 - y1) * (((double)i) / dx));
                vx[i] = Math.sqrt((x - x1)*(x - x1) + (y - y1)*(y - y1));
                vy[i] = valeurs[x][y];
            }
        } else {
            vx = new double[dy];
            vy = new double[dy];
            int sgn;
            if (y2 > y1)
                sgn = 1;
            else
                sgn = -1;
            for (int i=0; i<dy; i++) {
                x = x1 + (int)Math.round((x2 - x1) * (((double)i) / dy));
                y = y1 + sgn * i;
                vx[i] = Math.sqrt((x - x1)*(x - x1) + (y - y1)*(y - y1));
                vy[i] = valeurs[x][y];
            }
        }
        (new FenetreGraphe(vx, vy, "x", "y")).show();
    }
    
    protected void filtre() {
        FiltreDlg f = new FiltreDlg(chercherFrame());
        Point apppoint = getLocationOnScreen();
        f.setLocation(apppoint.x+50, apppoint.y+50);
        f.setVisible(true);
        int[][] filtre = f.getFiltre();
        if (filtre == null)
            return;
        int[][] valeurs = imageAffichee.getValeurs();
        int dimx = valeurs.length;
        int dimy = valeurs[0].length;
        valeursCalcul = new int[dimx][dimy];
        int dimf = filtre.length;
        double sommef;
        double sommev;
        int px, py;
        for (int i=0; i<dimx; i++)
            for (int j=0; j<dimy; j++) {
                sommef = 0;
                sommev = 0;
                for (int k=0; k<dimf; k++)
                    for (int l=0; l<dimf; l++) {
                        px = i + k - dimf/2;
                        py = j + l - dimf/2;
                        if (px > 0 && px < dimx && py > 0 && py < dimy) {
                            sommef += filtre[k][l];
                            sommev += valeurs[px][py] * filtre[k][l];
                        }
                    }
                valeursCalcul[i][j] = (int)Math.round(sommev / sommef);
            }
        String nomNouvelleImage = demander(rb.getString("Nom_nouvelle_image"));
        if (nomNouvelleImage == null) {
            valeursCalcul = null;
            return;
        }
        ImageMemoire nouvelleImage = new ImageMemoire(nomNouvelleImage, valeursCalcul, this);
        nouvelleImage.setSeuils(imageAffichee.getSeuilInf(), imageAffichee.getSeuilSup());
        listeImages.addElement(nouvelleImage);
        imgList.add(nouvelleImage.getNom());
        imgList.select(imgList.getItemCount()-1);
        ouvrir(nouvelleImage);
        valeursCalcul = null;
    }
    
    protected void supprimer() {
        indices = selectionImages(rb.getString("Slectionnez_images_supprimer"), 1, listeImages.size());
        if (indices == null)
            return;
        boolean changer = false;
        for (int i=0; i<indices.length; i++)
            if ((ImageIntf)listeImages.elementAt(indices[i]) == imageAffichee)
                changer = true;
        for (int i=0; i<indices.length; i++) {
            imgList.remove(indices[i]);
            listeImages.removeElementAt(indices[i]);
            for (int j=i+1; j<indices.length; j++)
                if (indices[j] > indices[i])
                    indices[j] --;
        }
        if (changer) {
            imgList.select(0);
            ouvrir((ImageIntf)listeImages.elementAt(0));
        }
    }
}
