/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package traitement;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/**
 * Zone d'affichage d'une image
 */
class ImageCanvas extends Canvas implements MouseMotionListener, MouseListener, KeyListener {
    public final static int RIEN = 0;
    public final static int POINT = 1;
    public final static int LIGNE = 2;
    public final static int CERCLE = 3;
    
    Traitement app;
    Image image;
    Dimension size;
    int w, h;
    int outil;
    Point p1 = null, p2 = null;
    
    // on suppose ici que l'image est dj charge
    public ImageCanvas(Image image, Traitement app) {
        
        if (image == null) {
            System.err.println("ImageCanvas: erreur: image == null");
            return;
        }

        this.image = image;
        this.app = app;
        
        w = image.getWidth(null);
        h = image.getHeight(null);
        
        if (w == -1 || h == -1) {
            System.err.println("ImageCanvas: erreur: image non charge");
            w = 100;
            h = 100;
        }
        
        size = new Dimension(w,h);
        
        setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        addMouseMotionListener(this);
        addMouseListener(this);
        addKeyListener(this);
    }

    public void changerImage(Image im) {
        image = im;
        repaint();
    }
    
    public Dimension getPreferredSize() {
        return getMinimumSize();
    }

    public Dimension getMinimumSize() {
        return size;
    }

    public void paint (Graphics g) {
        g.drawImage(image, 0, 0, this);
        //g.drawRect(0, 0, w - 1, h - 1);
        
        dessinerObjet();
    }
    
    public void mouseMoved(MouseEvent e) {
        //String slab = "coordonnes :  " + e.getX() + ", " + e.getY();
        //labcoords.setText(slab);
    }
    
    public void mouseDragged(MouseEvent e) {
        if (outil == LIGNE && p1 != null) {
            boolean nouveau = (p2 == null);
            if (p2 != null)
                dessinerLigne();
            p2 = e.getPoint();
            dessinerLigne();
            double l = Math.sqrt(Math.pow(p2.x - p1.x, 2) +
                Math.pow(p2.y - p1.y, 2));
            l = Math.rint(l*10)/10;
        } else if (outil == CERCLE && p1 != null) {
            boolean nouveau = (p2 == null);
            if (p2 != null)
                dessinerCercle();
            p2 = e.getPoint();
            dessinerCercle();
        }
    }
    
    public void mouseClicked(MouseEvent e) {
        if (outil == POINT) {
            if (p1 != null)
                dessinerPoint();
            p1 = e.getPoint();
            dessinerPoint();
        }
    }
    
    public void mouseEntered(MouseEvent e) {
    }
    
    public void mouseExited(MouseEvent e) {
    }
    
    public void mousePressed(MouseEvent e) {
        if (outil == LIGNE) {
            if (p2 != null) {
                dessinerLigne();
                p2 = null;
            }
            p1 = e.getPoint();
        } else if (outil == CERCLE) {
            if (p2 != null) {
                dessinerCercle();
                p2 = null;
            }
            p1 = e.getPoint();
        }
    }
    
    public void mouseReleased(MouseEvent e) {
        if (outil == LIGNE) {
            if (p2 != null)
                dessinerLigne();
            p2 = e.getPoint();
            dessinerLigne();
        }
        app.suiteOperation(p1, p2);
    }
    
    public void keyPressed(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_LEFT: deplacer(-1, 0); break;
            case KeyEvent.VK_RIGHT: deplacer(1, 0); break;
            case KeyEvent.VK_UP: deplacer(0, -1); break;
            case KeyEvent.VK_DOWN: deplacer(0, 1); break;
            case KeyEvent.VK_ADD: agrandir(1); break;
            case KeyEvent.VK_SUBTRACT: agrandir(-1); break;
            default: ;
        }
    }
    
    public void keyReleased(KeyEvent e) {
    }
    
    public void keyTyped(KeyEvent e) {
    }
    
    public void deplacer(int dx, int dy) {
        dessinerObjet();
        if (p1 != null) {
            p1.x += dx;
            p1.y += dy;
        }
        if (p2 != null) {
            p2.x += dx;
            p2.y += dy;
        }
        dessinerObjet();
    }
    
    public void agrandir(int dr) {
        if (outil == CERCLE && p1 != null && p2 != null) {
            dessinerCercle();
            if (p2.x > p1.x)
                p2.x += dr;
            else
                p2.x -= dr;
            if (p2.y > p1.y)
                p2.y += dr;
            else
                p2.y -= dr;
            dessinerCercle();
        }
    }
    
    public void setOutil(int o) {
        if (outil == POINT && p1 != null)
            dessinerPoint();
        else if (outil == LIGNE && p2 != null)
            dessinerLigne();
        else if (outil == CERCLE && p2 != null)
            dessinerCercle();
        outil = o;
        p1 = null;
        p2 = null;
    }
    
    public void dessinerObjet() {
        if (outil == POINT && p1 != null)
            dessinerPoint();
        if (outil == LIGNE && p1 != null && p2 != null)
            dessinerLigne();
        if (outil == CERCLE && p1 != null && p2 != null)
            dessinerCercle();
    }
    
    public void dessinerPoint() {
        Graphics g = getGraphics();
        g.setXORMode(Color.white);
        g.drawLine(p1.x - 5, p1.y, p1.x + 5, p1.y);
        g.drawLine(p1.x, p1.y - 5, p1.x, p1.y + 5);
        g.setPaintMode();
    }
    
    public void dessinerLigne() {
        Graphics g = getGraphics();
        g.setXORMode(Color.white);
        g.drawLine(p1.x, p1.y, p2.x, p2.y);
        g.setPaintMode();
    }
    
    public void dessinerCercle() {
        Graphics g = getGraphics();
        g.setXORMode(Color.white);
        int rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
        g.drawOval(p1.x - rayon, p1.y - rayon, rayon*2, rayon*2);
        g.setPaintMode();
    }
    
    /*public void changeImage(Image im) {
        tracker.removeImage(image,0);
        this.image = im;
        tracker.addImage(image, 0);
    }*/
}
