/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

function Traces(app, divimage, width, height) {
    var that;
    
    this.app = app;
    this.width = width;
    this.height = height;
    this.outil = 0;
    this.p1 = null;
    this.p2 = null;
    this.drag = false;
    
    this.canvas = document.createElement('canvas');
    this.canvas.setAttribute('width', width);
    this.canvas.setAttribute('height', height);
    this.canvas.style.position = 'absolute';
    this.canvas.style.left = '0px';
    this.canvas.style.top = '0px';
    divimage.appendChild(this.canvas);
    this.ctx = this.canvas.getContext('2d');
    that = this;
    this.mousedownProxy = function(e) { that.mousedown(e) };
    this.canvas.addEventListener('mousedown', this.mousedownProxy, false);
    this.mousemoveProxy = function(e) { that.mousemove(e) };
    this.canvas.addEventListener('mousemove', this.mousemoveProxy, false);
    this.mouseupProxy = function(e) { that.mouseup(e) };
    this.canvas.addEventListener('mouseup', this.mouseupProxy, false);
    this.mouseclickProxy = function(e) { that.mouseclick(e) };
    this.canvas.addEventListener('click', this.mouseclickProxy, false);
    if ('ontouchstart' in document) {
        this.canvas.addEventListener('touchstart', this.mousedownProxy, false);
        this.canvas.addEventListener('touchmove', this.mousemoveProxy, false);
        this.canvas.addEventListener('touchend', this.mouseupProxy, false);
    }
}

Traces.RIEN = 0;
Traces.POINT = 1;
Traces.LIGNE = 2;
Traces.CERCLE = 3;

Traces.prototype.mousemove = function(e) {
    if (this.drag) {
        if (this.outil == Traces.LIGNE && this.p1 != null) {
            this.ctx.clearRect(0, 0, this.width, this.height);
            this.p2 = Traitement.coordonnees(e, this.canvas);
            this.dessinerLigne(this.ctx, this.p1, this.p2);
        } else if (this.outil == Traces.CERCLE && this.p1 != null) {
            this.ctx.clearRect(0, 0, this.width, this.height);
            this.p2 = Traitement.coordonnees(e, this.canvas);
            this.dessinerCercle(this.ctx, this.p1, this.p2);
        }
    } else {
        this.app.mousemove(e);
    }
}

Traces.prototype.mouseclick = function(e) {
    if (this.outil == Traces.POINT) {
        this.p1 = Traitement.coordonnees(e, this.canvas);
        this.dessinerPoint(this.ctx, this.p1, this.p2);
    }
}

Traces.prototype.mousedown = function(e) {
    this.drag = true;
    if (this.outil == Traces.LIGNE || this.outil == Traces.CERCLE) {
        this.p1 = Traitement.coordonnees(e, this.canvas);
        if (e.preventDefault)
            e.preventDefault();
        else
            e.returnValue = false;
    }
}

Traces.prototype.mouseup = function(e) {
    this.drag = false;
    this.ctx.clearRect(0, 0, this.width, this.height);
    this.app.suiteOperation(this.p1, this.p2);
    this.p1 = null;
    this.p2 = null;
}

Traces.prototype.setOutil = function(o) {
    this.outil = o;
    this.p1 = null;
    this.p2 = null;
}

Traces.prototype.dessinerObjet = function(ctx) {
    if (this.outil == Traces.POINT && this.p1 != null)
        this.dessinerPoint(ctx, this.p1, this.p2);
    else if (this.outil == Traces.LIGNE && this.p1 != null && this.p2 != null)
        this.dessinerLigne(ctx, this.p1, this.p2);
    else if (this.outil == Traces.CERCLE && this.p1 != null && this.p2 != null)
        this.dessinerCercle(ctx, this.p1, this.p2);
}

Traces.prototype.dessinerPoint = function(ctx, p1, p2) {
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.moveTo(p1.x - 5 + 0.5, p1.y + 0.5);
    ctx.lineTo(p1.x + 5 + 0.5, p1.y + 0.5);
    ctx.moveTo(p1.x + 0.5, p1.y - 5 + 0.5);
    ctx.lineTo(p1.x + 0.5, p1.y + 5 + 0.5);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.moveTo(p1.x - 0.5 - 5, p1.y - 0.5);
    ctx.lineTo(p1.x - 0.5 + 5, p1.y - 0.5);
    ctx.moveTo(p1.x - 0.5, p1.y - 5 - 0.5);
    ctx.lineTo(p1.x - 0.5, p1.y + 5 - 0.5);
    ctx.stroke();
}

Traces.prototype.dessinerLigne = function(ctx, p1, p2) {
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.moveTo(p1.x + 0.5, p1.y + 0.5);
    ctx.lineTo(p2.x + 0.5, p2.y + 0.5);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.moveTo(p1.x - 0.5, p1.y - 0.5);
    ctx.lineTo(p2.x - 0.5, p2.y - 0.5);
    ctx.stroke();
}

Traces.prototype.dessinerCercle = function(ctx, p1, p2) {
    var rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.arc(p1.x + 0.5, p1.y + 0.5, rayon, 0, 2*Math.PI, false);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.arc(p1.x - 0.5, p1.y - 0.5, rayon, 0, 2*Math.PI, false);
    ctx.stroke();
}
