/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/



Traitement.lireParametres = function() {
    var parametres, searchString, params, i, nomval;
    
    parametres = new Array();
    searchString = window.location.search.substring(1);
    params = searchString.split("&");
    
    for (i=0; i<params.length; i++) {
        nomval = params[i].split("=");
        // remarque: les paramètres de l'URL doivent être encodés en UNICODE
        parametres[nomval[0]] = decodeURIComponent(nomval[1]);
    }
    return(parametres);
}

Traitement.isCanvasSupported = function() {
    var elem = document.createElement('canvas');
    return !!(elem.getContext && elem.getContext('2d'));
}

Traitement.versionApplet = function() {
    var applet, width, height, parametres, chemin, nom, param;
    
    if (document.body.firstChild != null) {
        while (document.body.hasChildNodes())
            document.body.removeChild(document.body.lastChild);
    }
    applet = document.createElement('applet');
    if (typeof(window.innerWidth) == 'number') {
        // autre que IE
        width = window.innerWidth;
        height = window.innerHeight;
    } else if ( document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight) ) {
        // IE 6+ en mode 'standards compliant'
        width = document.documentElement.clientWidth;
        height = document.documentElement.clientHeight;
    } else {
        width = '100%';
        height = '100%';
    }
    applet.setAttribute('width', width);
    applet.setAttribute('height', height);
    applet.setAttribute('archive', 'Traitement.jar');
    applet.setAttribute('code', 'traitement.Traitement');
    
    parametres = Traitement.lireParametres();
    chemin = parametres['chemin_fichiers'];
    if (chemin == undefined)
        chemin = '';
    for (nom in parametres) {
        if (parametres.hasOwnProperty(nom)) {
            param = document.createElement('param');
            param.setAttribute('name', nom);
            param.setAttribute('value', chemin + parametres[nom]);
            applet.appendChild(param);
        }
    }
    
    document.body.appendChild(applet);
}

Traitement.tableau2D = function(x, y) {
    var t, i, j;
    t = new Array(x);
    for (i=0; i<x; i++)
        t[i] = new Array(y);
    for (i=0; i<x; i++)
        for (j=0; j<y; j++)
            t[i][j] = 0;
    return(t);
}

Traitement.coordonnees = function(e, currentElement) {
    // résultat décalé de 1px avec excanvas :(
    var x, y, totalOffsetX, totalOffsetY;
    
    if (e.pageX || e.pageY) {
      x = e.pageX;
      y = e.pageY;
    } else {
      x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
      y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    }
    totalOffsetX = 0;
    totalOffsetY = 0;
    do {
        totalOffsetX += currentElement.offsetLeft;
        totalOffsetY += currentElement.offsetTop;
    } while (currentElement = currentElement.offsetParent)
    x -= totalOffsetX;
    y -= totalOffsetY;
    return( { x : Math.round(x), y : Math.round(y) } );
}



new Traitement();


function Traitement() {
    var that;
    
    if (Traitement.isCanvasSupported()) {
        this.imgList = null; // List -> select
        this.listeImages = null; // tableau de ImageIntf
        this.imageAffichee = null; // ImageIntf
        this.tfSeuilBas = null; // TextField -> input
        this.tfSeuilHaut = null; // TextField -> input
        this.operations1 = null; // select
        this.operations2 = null; // select
        this.palChoice = null; // Choice -> select
        this.canvas = null; // ImageCanvas -> canvas
        this.traces = null; // Traces
        this.ctx = null;
        this.coordLabel = null; // Label
        this.infoLabel = null; // Label
        this.opCourante = null; // String
        this.numImageCourante = -1; // int
        this.indices = null; // int[]
        this.valeursCalcul = null; // int[][]
        this.ptRef = null; // Point
        this.bchanger = null;
        this.bauto = null; // Button
        this.palette = null; // int[][]
        
        that = this;
        this.proxyChangerSeuils = function(e) { e.preventDefault(); that.changerSeuils(); return(false); };
        this.proxySeuilsAuto = function(e) { that.seuilsAuto(); };
        window.addEventListener('load', function() { that.init(); }, false);
        
    } else {
        // on ne peut pas utiliser excanvas ou flashcanvas pour manipuler des bitmaps offscreen
        // donc on utilise la version sous forme d'applet en alternative pour les vieux navigateurs
        if (window.addEventListener)
            window.addEventListener('load', function() { Traitement.versionApplet(); }, false);
        else if (window.attachEvent)
            window.attachEvent('onload', function() { Traitement.versionApplet(); });
    }
}

Traitement.prototype.afficherErreur = function(ex) {
    var s;
    if (ex.message)
        s = ex.message;
    else
        s = ex;
    if (ex.fileName)
        s += ' in ' + ex.fileName;
    if (ex.lineNumber)
        s += ' line ' + ex.lineNumber;
    alert(Messages.get('Erreur') + ': ' + s);
}

Traitement.prototype.init = function() {
    var parametres, chemin, surl, n, i, html_select, that, div_liste_images, div_image, image, div_palette, div_seuils, form_seuils;
    
    parametres = Traitement.lireParametres();
    chemin = parametres['chemin_fichiers'];
    if (chemin == undefined)
        chemin = '';
    
    div_operations = document.getElementById('div-operations');
    /*
    on ne peut pas changer directement le select avec ie
    cf bug http://support.microsoft.com/kb/276228
    */
    html_select = '';
    html_select = '<select id="operations1">';
    html_select += '<option>' + Messages.get("opération_sur_image_sélectionnée") + '</option>';
    html_select += '<option value="coupe">' + Messages.get("coupe") + '</option>';
    html_select += '<option value="filtre">' + Messages.get("filtre") + '</option>';
    html_select += '</select>';
    html_select += '<select id="operations2">';
    html_select += '<option>' + Messages.get("opération_sur_plusieurs_images") + '</option>';
    html_select += '<option value="addition_avec_recentrage">' + Messages.get("addition_avec_recentrage") + '</option>';
    html_select += '<option value="addition_sans_recentrage">' + Messages.get("addition_sans_recentrage") + '</option>';
    html_select += '<option value="soustraction">' + Messages.get("soustraction") + '</option>';
    html_select += '<option value="division">' + Messages.get("division") + '</option>';
    html_select += '<option value="suppression">' + Messages.get("suppression") + '</option>';
    html_select += '</select>';
    div_operations.innerHTML = html_select;
    this.operations1 = document.getElementById('operations1');
    that = this;
    this.operations1.onchange = function() { that.operation(this.options[this.selectedIndex].value); this.selectedIndex = 0; };
    this.operations2 = document.getElementById('operations2');
    this.operations2.onchange = function() { that.operation(this.options[this.selectedIndex].value); this.selectedIndex = 0; };
    
    this.creerPalette("niveaux_de_gris");
    n = 0;
    this.listeImages = new Array();
    while (parametres["image" + (n+1)] != undefined) {
        n++;
        try {
            this.listeImages.push(new ImageFITS(chemin + parametres["image" + n], this.palette));
        } catch (ex) {
            this.afficherErreur(ex);
            n--;
            break;
        }
    }
    
    if (this.listeImages.length == 0) {
        alert("Erreur: aucune image en paramètre !");
        return;
    }
    
    div_liste_images = document.getElementById('div-liste-images');
    html_select = '<select id="liste-images" size="14">';
    for (i=0; i<this.listeImages.length; i++) {
        html_select += '<option>' + this.listeImages[i].getNom() + '</option>';
    }
    html_select += '</select>';
    div_liste_images.innerHTML = html_select;
    this.imgList = document.getElementById('liste-images');
    this.imgList.onchange = function() { that.selection() };
    
    this.div_image = document.getElementById('div-image');
    image = this.listeImages[0].getImage(); // ImageData
    this.div_image.style.width = image.width + 'px';
    this.div_image.style.height = image.height + 'px';
    this.canvas = document.createElement('canvas');
    this.canvas.setAttribute('width', image.width);
    this.canvas.setAttribute('height', image.height);
    this.div_image.appendChild(this.canvas);
    if (!this.canvas.getContext) {
        document.getElementById('div-message').innerHTML = '<p>Votre navigateur ne gère pas HTML5.</p>';
        return;
    }
    this.ctx = this.canvas.getContext('2d');
    this.ctx.putImageData(image, 0, 0);
    this.traces = new Traces(this, this.div_image, image.width, image.height);
    
    this.coordLabel = document.getElementById('div-pixel');
    
    div_palette = document.getElementById('div-palette');
    html_select = Messages.get("palette") + ' ';
    html_select += '<select id="palette">';
    html_select += '<option value="niveaux_de_gris">' + Messages.get("niveaux_de_gris") + '</option>';
    html_select += '<option value="arc-en-ciel">' + Messages.get("arc-en-ciel") + '</option>';
    html_select += '<option value="rouges">' + Messages.get("rouges") + '</option>';
    html_select += '<option value="verts">' + Messages.get("verts") + '</option>';
    html_select += '<option value="bleus">' + Messages.get("bleus") + '</option>';
    html_select += '</select>';
    div_palette.innerHTML = html_select;
    this.palChoice = document.getElementById('palette');
    this.palChoice.onchange = function() { that.changerPalette(this.options[this.selectedIndex].value) };
    
    div_seuils = document.getElementById('div-seuils');
    form_seuils = document.createElement('form');
    form_seuils.addEventListener('submit', this.proxyChangerSeuils, false);
    form_seuils.setAttribute('autocomplete', 'off');
    form_seuils.appendChild(document.createTextNode(Messages.get('seuil_bas')));
    this.tfSeuilBas = document.createElement('input');
    this.tfSeuilBas.setAttribute('type', 'text');
    this.tfSeuilBas.style.width = '4em';
    form_seuils.appendChild(this.tfSeuilBas);
    form_seuils.appendChild(document.createTextNode(Messages.get('seuil_haut')));
    this.tfSeuilHaut = document.createElement('input');
    this.tfSeuilHaut.setAttribute('type', 'text');
    this.tfSeuilHaut.style.width = '4em';
    form_seuils.appendChild(this.tfSeuilHaut);
    this.bchanger = document.createElement('button');
    this.bchanger.setAttribute('id', 'seuils');
    this.bchanger.setAttribute('type', 'submit');
    this.bchanger.setAttribute('value', Messages.get('changer'));
    this.bchanger.appendChild(document.createTextNode(Messages.get('changer')));
    form_seuils.appendChild(this.bchanger);
    this.bauto = document.createElement('button');
    this.bauto.setAttribute('id', 'auto');
    this.bauto.setAttribute('type', 'button');
    this.bauto.appendChild(document.createTextNode(Messages.get('auto')));
    this.bauto.addEventListener('click', this.proxySeuilsAuto, false);
    form_seuils.appendChild(this.bauto);
    div_seuils.appendChild(form_seuils);
    
    this.infoLabel = document.getElementById('div-message');
    
    // les évènements mousemove sont gérés dans Traces, à cause du canvas par-dessus
    
    if (this.listeImages.length > 0) {
        this.imgList.selectedIndex = 0;
        this.ouvrir(this.listeImages[0]);
    }
}

Traitement.prototype.mousemove = function(e) {
    var valeurs, dimx, dimy, pt, texte, i;
    
    valeurs = this.imageAffichee.getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    pt = Traitement.coordonnees(e, this.canvas);
    if (pt.x >= 0 && pt.x < dimx && pt.y >= 0 && pt.y < dimy) {
        texte = "("+(pt.x+1)+","+(pt.y+1)+") : " + valeurs[pt.x][pt.y];
        if (this.coordLabel.firstChild == null)
            this.coordLabel.appendChild(document.createTextNode(texte));
        else
            this.coordLabel.firstChild.nodeValue = texte;
    }
}

Traitement.prototype.info = function(msg) {
    if (this.infoLabel.firstChild == null)
        this.infoLabel.appendChild(document.createTextNode(msg));
    else
        this.infoLabel.firstChild.nodeValue = msg;
}

Traitement.prototype.ouvrir = function(im) {
    var imgdata;
    
    this.imageAffichee = im;
    this.imageAffichee.setPalette(this.palette);
    imgdata = this.imageAffichee.getImage();
    this.ctx.putImageData(imgdata, 0, 0);
    this.tfSeuilBas.value = '' + this.imageAffichee.getSeuilInf();
    this.tfSeuilHaut.value = '' + this.imageAffichee.getSeuilSup();
}

Traitement.prototype.operation = function(nomOp) {
    if (nomOp == "coupe")
        this.coupe();
    else if (nomOp == "filtre")
        this.filtre();
    else if (nomOp == "addition_avec_recentrage")
        this.additionAvecRecentrage();
    else if (nomOp == "addition_sans_recentrage")
        this.additionSansRecentrage();
    else if (nomOp == "soustraction")
        this.soustraction();
    else if (nomOp == "division")
        this.division();
    else if (nomOp == "suppression")
        this.supprimer();
}

Traitement.prototype.suiteOperation = function(pt1, pt2) {
    if (this.opCourante == "addition")
        this.suiteAddition(pt1, pt2);
    else if (this.opCourante == "coupe")
        this.suiteCoupe(pt1, pt2);
}

Traitement.prototype.creerPalette = function(choix) {
    var i;
    
    if (this.palette == null)
        this.palette = Traitement.tableau2D(256, 3);
    if (choix == "niveaux_de_gris") {
        for (i=0; i<256; i++) {
            this.palette[i][0] = i;
            this.palette[i][1] = i;
            this.palette[i][2] = i;
        }
    } else if (choix == "arc-en-ciel") {
        // violet bleu vert jaune orange rouge
        for (i=0; i<51; i++) {
            this.palette[i][0] = 102 - 2*i;
            this.palette[i][1] = 0;
            this.palette[i][2] = 255;
        }
        for (i=51; i<102; i++) {
            this.palette[i][0] = 0;
            this.palette[i][1] = (i - 51)*5;
            this.palette[i][2] = 255 - (i - 51)*5;
        }
        for (i=102; i<153; i++) {
            this.palette[i][0] = (i - 102)*5;
            this.palette[i][1] = 255;
            this.palette[i][2] = 0;
        }
        for (i=153; i<204; i++) {
            this.palette[i][0] = 255;
            this.palette[i][1] = 255 - 2*(i - 153);
            this.palette[i][2] = 0;
        }
        for (i=204; i<255; i++) {
            this.palette[i][0] = 255;
            this.palette[i][1] = 153 - 3*(i - 204);
            this.palette[i][2] = 0;
        }
        this.palette[255][0] = 255;
        this.palette[255][1] = 0;
        this.palette[255][2] = 0;
    } else if (choix == "rouges") {
        for (i=0; i<256; i++) {
            this.palette[i][0] = i;
            this.palette[i][1] = 0;
            this.palette[i][2] = 0;
        }
    } else if (choix == "verts") {
        for (i=0; i<256; i++) {
            this.palette[i][0] = 0;
            this.palette[i][1] = i;
            this.palette[i][2] = 0;
        }
    } else if (choix == "bleus") {
        for (i=0; i<256; i++) {
            this.palette[i][0] = 0;
            this.palette[i][1] = 0;
            this.palette[i][2] = i;
        }
    }
}

Traitement.prototype.changerPalette = function(choix) {
    var i, img;
    
    this.creerPalette(choix);
    this.imageAffichee.setPalette(this.palette);
    img = this.imageAffichee.getImage();
    this.ctx.putImageData(img, 0, 0);
}

Traitement.prototype.getPalette = function() {
    return(this.palette);
}

Traitement.prototype.changerSeuils = function() {
    var sbas, shaut, bas, haut, img;
    
    sbas = this.tfSeuilBas.value;
    shaut = this.tfSeuilHaut.value;
    bas = parseInt(sbas);
    haut = parseInt(shaut);
    this.imageAffichee.setSeuils(bas, haut);
    img = this.imageAffichee.getImage();
    this.ctx.putImageData(img, 0, 0);
}

Traitement.prototype.seuilsAuto = function() {
    var valeurs, dimx, dimy, mini, maxi, i, j, dimc, comptage, posc, nb, imini, bas, imaxi, haut;
    
    valeurs = this.imageAffichee.getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    mini = Number.MAX_VALUE;
    maxi = 0;
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++) {
            if (valeurs[i][j] < mini)
                mini = valeurs[i][j];
            if (valeurs[i][j] > maxi)
                maxi = valeurs[i][j];
        }
    
    dimc = 100;
    comptage = new Array(dimc);
    for (i=0; i<dimc; i++)
        comptage[i] = 0;
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++) {
            posc = Math.round((dimc - 1) * (Math.log(valeurs[i][j] - mini + 1) / Math.log(maxi - mini + 1)));
            comptage[posc] = comptage[posc] + 1;
        }
    nb = 0;
    imini = 0;
    while (nb < (dimx * dimy)*0.04) {
        nb += comptage[imini];
        imini++;
    }
    bas = Math.round(mini + Math.exp((imini/(dimc - 1)) * Math.log(maxi - mini)));
    nb = 0;
    imaxi = 0;
    while (nb < (dimx * dimy)*0.99) {
        nb += comptage[imaxi];
        imaxi++;
    }
    haut = Math.round(mini + Math.exp((imaxi/(dimc - 1)) * Math.log(maxi - mini)));
    if (haut > maxi) // pas terrible cet algo...
        haut = maxi;
    this.tfSeuilBas.value = '' + bas;
    this.tfSeuilHaut.value = '' + haut;
    this.imageAffichee.setSeuils(bas, haut);
    this.ctx.putImageData(this.imageAffichee.getImage(), 0, 0);
}

Traitement.prototype.selection = function() {
    this.ouvrir(this.listeImages[this.imgList.selectedIndex]);
}

Traitement.prototype.alerte = function(message) {
    alert(message);
}

Traitement.prototype.demander = function(message) {
    return(prompt(message, ''));
}

Traitement.prototype.selectionImages = function(question, minsel, maxsel, fct_suite) {
    var dlg;
    
    dlg = new SelectionDlg(question, minsel, maxsel, this.listeImages, fct_suite);
    dlg.afficher(); // dlg doit appeler fct_suite avec les indices en paramètre quand le dialogue est validé
}

Traitement.prototype.additionAvecRecentrage = function() {
    var that = this;
    this.selectionImages(Messages.get("Sélectionnez_addition"), 2, this.listeImages.length, function(indices) {
        that.fct_additionAvecRecentrage(indices);
    });
}

Traitement.prototype.fct_additionAvecRecentrage = function(indices) {
    this.indices = indices;
    if (indices == null)
        return;
    this.info(Messages.get("Sélectionnez_étoile"));
    this.traces.setOutil(Traces.CERCLE);
    this.opCourante = 'addition';
    this.numImageCourante = 0;
    this.bchanger.disabled = true;
    this.ouvrir(this.listeImages[indices[this.numImageCourante]]);
}

Traitement.prototype.suiteAddition = function(p1, p2) {
    var valeurs, dimx, dimy, i, j, nouvPt, i2, j2;
    
    valeurs = this.imageAffichee.getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    if (this.numImageCourante == 0) {
        this.ptRef = this.baryDisque(p1, p2, valeurs);
        this.valeursCalcul = Traitement.tableau2D(dimx, dimy);
        for (i=0; i<dimx; i++)
            for (j=0; j<dimy; j++)
                this.valeursCalcul[i][j] = valeurs[i][j];
    } else {
        nouvPt = this.baryDisque(p1, p2, valeurs);
        for (i=0; i<dimx; i++)
            for (j=0; j<dimy; j++) {
                i2 = i + (nouvPt.x - this.ptRef.x);
                j2 = j + (nouvPt.y - this.ptRef.y);
                if (i2 >= 0 && i2 < dimx && j2 >= 0 && j2 < dimy)
                    this.valeursCalcul[i][j] += valeurs[i2][j2];
            }
    }
    this.numImageCourante++;
    if (this.numImageCourante < this.indices.length) {
        this.imgList.selectedIndex = this.indices[this.numImageCourante];
        this.ouvrir(this.listeImages[this.indices[this.numImageCourante]]);
        this.traces.setOutil(Traces.CERCLE);
    } else
        this.finAddition();
}

Traitement.prototype.additionSansRecentrage = function() {
    var that = this;
    this.selectionImages(Messages.get("Sélectionnez_addition"), 2, this.listeImages.length, function(indices) {
        that.fct_additionSansRecentrage(indices);
    });
}

Traitement.prototype.fct_additionSansRecentrage = function(indices) {
    var valeurs, dimx, dimy, i, j;
    
    this.indices = indices;
    if (indices == null)
        return;
    valeurs = (this.listeImages[indices[0]]).getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    this.valeursCalcul = Traitement.tableau2D(dimx, dimy);
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++)
            this.valeursCalcul[i][j] = valeurs[i][j];
    for (this.numImageCourante=1; this.numImageCourante < indices.length; this.numImageCourante++) {
        valeurs = (this.listeImages[indices[this.numImageCourante]]).getValeurs();
        for (i=0; i<dimx; i++)
            for (j=0; j<dimy; j++)
                this.valeursCalcul[i][j] += valeurs[i][j];
    }
    this.finAddition();
}

// calcul du barycentre d'un ensemble de points à l'intérieur d'un disque définit par son centre p1 et un point p2
// renvoie un point (x, y)
// on prend en fait le barycentre des valeurs - minimum au carré pour mieux repérer le centre des astres
Traitement.prototype.baryDisque = function(p1, p2, valeurs) {
    var dimx, dimy, rayon, xmin, xmax, ymin, ymax, vx, vy, total, i, j, xres, yres, rayon2, v;
    
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
    xmin = p1.x - rayon;
    if (xmin < 0)
        xmin = 0;
    xmax = p1.x + rayon;
    if (xmax > dimx-1)
        xmax = dimx-1;
    ymin = p1.y - rayon;
    if (ymin < 0)
        ymin = 0;
    ymax = p1.y + rayon;
    if (ymax > dimy-1)
        ymax = dimy-1;
    rayon2 = rayon*rayon;
    minimum = Number.MAX_VALUE;
    for (i=xmin; i<xmax; i++)
        for (j=ymin; j<ymax; j++)
            if (Math.pow(i - p1.x, 2) + Math.pow(j - p1.y, 2) <= rayon2) {
                if (valeurs[i][j] < minimum)
                    minimum = valeurs[i][j];
            }
    vx = 0;
    vy = 0;
    total = 0;
    for (i=xmin; i<xmax; i++)
        for (j=ymin; j<ymax; j++)
            if (Math.pow(i - p1.x, 2) + Math.pow(j - p1.y, 2) <= rayon2) {
                v = Math.pow(valeurs[i][j] - minimum, 2);
                vx += i*v;
                vy += j*v;
                total += v;
            }
    xres = Math.round(vx / total);
    yres = Math.round(vy / total);
    return({x: xres, y: yres});
}

Traitement.prototype.finAddition = function() {
    var dimx, dimy, i, j, nomNouvelleImage, nouvelleImage;
    
    this.info('');
    this.bchanger.disabled = false;
    this.opCourante = null;
    this.numImageCourante = -1;
    this.traces.setOutil(Traces.RIEN);
    dimx = this.valeursCalcul.length;
    dimy = this.valeursCalcul[0].length;
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++)
            this.valeursCalcul[i][j] = Math.floor(this.valeursCalcul[i][j] / this.indices.length);
    nomNouvelleImage = this.demander(Messages.get("Nom_nouvelle_image"));
    if (nomNouvelleImage == null) {
        valeursCalcul = null;
        return;
    }
    nouvelleImage = new ImageMemoire(nomNouvelleImage, this.valeursCalcul, this.palette);
    nouvelleImage.setSeuils(this.imageAffichee.getSeuilInf(), this.imageAffichee.getSeuilSup());
    this.listeImages.push(nouvelleImage);
    this.imgList.options[this.imgList.options.length] = new Option(nouvelleImage.getNom(), nouvelleImage.getNom());
    this.imgList.selectedIndex = this.imgList.options.length - 1;
    this.ouvrir(nouvelleImage);
    
    this.valeursCalcul = null;
}

Traitement.prototype.soustraction = function() {
    var that = this;
    this.selectionImages(Messages.get("Sélectionnez_image1_soustraction"), 1, 1, function(indices) {
        var ind1;
        
        that.indices = indices;
        if (indices == null)
            return;
        ind1 = indices[0];
        that.selectionImages(Messages.get("Sélectionnez_image2"), 1, 1, function(indices) {
            if (indices == null)
                return;
            that.fct_soustraction(ind1, indices[0]);
        });
    });
}

Traitement.prototype.fct_soustraction = function(ind1, ind2) {
    var valeurs, dimx, dimy, i, j, nomNouvelleImage, nouvelleImage;
    
    valeurs = this.listeImages[ind1].getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    this.valeursCalcul = Traitement.tableau2D(dimx, dimy);
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++)
            this.valeursCalcul[i][j] = valeurs[i][j];
    valeurs = this.listeImages[ind2].getValeurs();
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++) {
            this.valeursCalcul[i][j] -= valeurs[i][j];
            if (this.valeursCalcul[i][j] < 0)
                this.valeursCalcul[i][j] = 0;
        }
    nomNouvelleImage = this.demander(Messages.get("Nom_nouvelle_image"));
    if (nomNouvelleImage == null) {
        this.valeursCalcul = null;
        return;
    }
    nouvelleImage = new ImageMemoire(nomNouvelleImage, this.valeursCalcul, this.palette);
    this.listeImages.push(nouvelleImage);
    this.imgList.options[this.imgList.options.length] = new Option(nouvelleImage.getNom(), nouvelleImage.getNom());
    this.imgList.selectedIndex = this.imgList.options.length - 1;
    this.ouvrir(nouvelleImage);
    this.valeursCalcul = null;
}

Traitement.prototype.division = function() {
    var that = this;
    this.selectionImages(Messages.get("Sélectionnez_image1_division"), 1, 1, function(indices) {
        var ind1;
        
        if (indices == null)
            return;
        ind1 = indices[0];
        that.selectionImages(Messages.get("Sélectionnez_image2"), 1, 1, function(indices) {
            if (indices == null)
                return;
            that.fct_division(ind1, indices[0]);
        });
    });
}

Traitement.prototype.fct_division = function(ind1, ind2) {
    var valeurs, dimx, dimy, i, j, total, moyenne, nomNouvelleImage, nouvelleImage;
    
    valeurs = this.listeImages[ind1].getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    this.valeursCalcul = Traitement.tableau2D(dimx, dimy);
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++)
            this.valeursCalcul[i][j] = valeurs[i][j];
    valeurs = this.listeImages[ind2].getValeurs();
    total = 0;
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++)
            total += valeurs[i][j];
    moyenne = total / (dimx * dimy);
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++) {
            if (valeurs[i][j] != 0)
                this.valeursCalcul[i][j] = Math.round(this.valeursCalcul[i][j] / (valeurs[i][j] / moyenne));
            else
                this.valeursCalcul[i][j] = 0;
        }
    nomNouvelleImage = this.demander(Messages.get("Nom_nouvelle_image"));
    if (nomNouvelleImage == null) {
        this.valeursCalcul = null;
        return;
    }
    nouvelleImage = new ImageMemoire(nomNouvelleImage, this.valeursCalcul, this.palette);
    nouvelleImage.setSeuils(this.imageAffichee.getSeuilInf(), this.imageAffichee.getSeuilSup());
    this.listeImages.push(nouvelleImage);
    this.imgList.options[this.imgList.options.length] = new Option(nouvelleImage.getNom(), nouvelleImage.getNom());
    this.imgList.selectedIndex = this.imgList.options.length - 1;
    this.ouvrir(nouvelleImage);
    this.valeursCalcul = null;
}

Traitement.prototype.coupe = function() {
    this.info(Messages.get("Sélectionnez_coupe"));
    this.traces.setOutil(Traces.LIGNE);
    this.opCourante = "coupe";
    this.bchanger.disabled = true;
}

Traitement.prototype.suiteCoupe = function(p1, p2) {
    var valeurs, dimx, dimy, x1, x2, y1, y2, dx, dy, vx, vy, x, y, sgn, i;
    
    this.info('');
    this.bchanger.disabled = false;
    this.opCourante = null;
    this.traces.setOutil(Traces.RIEN);
    valeurs = this.imageAffichee.getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    x1 = p1.x;
    x2 = p2.x;
    y1 = p1.y;
    y2 = p2.y;
    dx = Math.abs(x2 - x1);
    dy = Math.abs(y2 - y1);
    if (dx > dy) {
        vx = new Array(dx);
        vy = new Array(dx);
        if (x2 > x1)
            sgn = 1;
        else
            sgn = -1;
        for (i=0; i<dx; i++) {
            x = x1 + sgn * i;
            y = y1 + Math.round((y2 - y1) * (i / dx));
            vx[i] = Math.sqrt((x - x1)*(x - x1) + (y - y1)*(y - y1));
            vy[i] = valeurs[x][y];
        }
    } else {
        vx = new Array(dy);
        vy = new Array(dy);
        if (y2 > y1)
            sgn = 1;
        else
            sgn = -1;
        for (i=0; i<dy; i++) {
            x = x1 + Math.round((x2 - x1) * (i / dy));
            y = y1 + sgn * i;
            vx[i] = Math.sqrt((x - x1)*(x - x1) + (y - y1)*(y - y1));
            vy[i] = valeurs[x][y];
        }
    }
    (new FenetreGraphe(vx, vy, Messages.get("position"), Messages.get("intensité"))).afficher();
}

Traitement.prototype.filtre = function() {
    var that, dlg;
    
    that = this;
    dlg = new FiltreDlg(function(filtre) {
        that.fct_filtre(filtre);
    });
    dlg.afficher();
}

Traitement.prototype.fct_filtre = function(filtre) {
    var valeurs, dimx, dimy, dimf, sommef, sommev, px, py, i, j, k, l, nomNouvelleImage, nouvelleImage;
    
    if (filtre == null)
        return;
    valeurs = this.imageAffichee.getValeurs();
    dimx = valeurs.length;
    dimy = valeurs[0].length;
    this.valeursCalcul = Traitement.tableau2D(dimx, dimy);
    dimf = filtre.length;
    for (i=0; i<dimx; i++)
        for (j=0; j<dimy; j++) {
            sommef = 0;
            sommev = 0;
            for (k=0; k<dimf; k++) {
                for (l=0; l<dimf; l++) {
                    px = i + k - Math.floor(dimf/2);
                    py = j + l - Math.floor(dimf/2);
                    if (px > 0 && px < dimx && py > 0 && py < dimy) {
                        sommef += filtre[k][l];
                        sommev += valeurs[px][py] * filtre[k][l];
                    }
                }
            }
            if (sommef == 0 || (sommev > 0 && sommef < 0) || (sommev < 0 && sommef > 0))
                this.valeursCalcul[i][j] = 0;
            else
                this.valeursCalcul[i][j] = Math.round(sommev / sommef);
        }
    nomNouvelleImage = this.demander(Messages.get("Nom_nouvelle_image"));
    if (nomNouvelleImage == null) {
        this.valeursCalcul = null;
        return;
    }
    nouvelleImage = new ImageMemoire(nomNouvelleImage, this.valeursCalcul, this.palette);
    nouvelleImage.setSeuils(this.imageAffichee.getSeuilInf(), this.imageAffichee.getSeuilSup());
    this.listeImages.push(nouvelleImage);
    this.imgList.options[this.imgList.options.length] = new Option(nouvelleImage.getNom(), nouvelleImage.getNom());
    this.imgList.selectedIndex = this.imgList.options.length - 1;
    this.ouvrir(nouvelleImage);
    this.valeursCalcul = null;
}

Traitement.prototype.supprimer = function() {
    var that = this;
    this.selectionImages(Messages.get("Sélectionnez_images_supprimer"), 1, this.listeImages.length, function(indices) {
        that.fct_supprimer(indices);
    });
}

Traitement.prototype.fct_supprimer = function(indices) {
    var changer, i, j;
    
    this.indices = indices;
    if (indices == null)
        return;
    changer = false;
    for (i=0; i<indices.length; i++)
        if (this.listeImages[indices[i]] == this.imageAffichee)
            changer = true;
    for (i=0; i<indices.length; i++) {
        this.imgList.remove(indices[i]);
        this.listeImages.splice(indices[i], 1);
        for (j=i+1; j<indices.length; j++)
            if (indices[j] > indices[i])
                indices[j] --;
    }
    if (changer) {
        this.imgList.selectedIndex = 0;
        this.ouvrir(this.listeImages[0]);
    }
}
